
het.tree.fast<-function(data, cc.var, T.vars, X.vars, Z.vars, G.vars, wts, run.options, Robj.file, myseed)
{
   run.options$mymethod <- list(eval=temp1, split=temp2, init=temp3) 

   set.seed(myseed)

   ## recode the genotype so that 0 is the common group

   g.v<-data[, G.vars]
   
   if (sum(g.v==0) < sum(g.v==2))
   {
     data[g.v==0, G.vars]<-2
     data[g.v==2, G.vars]<-0
   }

   result.obs<-fitmodel(data, cc.var, T.vars, X.vars, Z.vars, G.vars,
                      wts, run.options)

   dev.vec.obs<-result.obs$dev.vec	
   dev.mat<-permtest.vglm(data, cc.var, T.vars, X.vars, Z.vars, G.vars, 
                       wts, run.options)

   test.result<-get.pvalue(-1*dev.mat, -1*dev.vec.obs)
   
   obj<-list(result.obs=result.obs, dev.mat=dev.mat, dev.vec.obs=dev.vec.obs,
             test.result=test.result)
   save(obj, file=Robj.file)

   NULL
}
   




# History:  07/20/2006  Initial coding
#           07/21/2006  Make return vector have same order 

# A function to do a 'restricted' permutation

permut.res <- function(data.x) 
{

 data.x<-data.frame(data.x)
 n.X<-dim(data.x)[2]
 nobs <- nrow(data.x)

 data<-data.x
 for (i in 1:n.X)
 {
        data[,i]<-factor(data[,i])
 }
 
     # data  A data frame with all variables as factors
 
     # Get the interaction vector. Rows with the same levels will have
     #  the same value in the interaction vector. 
     inter <- as.numeric(interaction(data))

     # Get the unique integers in inter
     interu <- unique(inter)
 
     # Get the number of observations

     # Define the vector of ids, return list
     id.list    <- 1:nobs
     id.permute <- 1:nobs

     for (i in 1:length(interu)) {
       # Get the rows that have each interaction
         rows <- (inter == interu[i])

       # Put a permutation of these ids in the return list
       # Keep the same order.
       if (sum(rows) > 1) {
        id.permute[rows] <- sample(id.list[rows])
       }    
  }
 
 
 id.permute
}


restr.outcome<-function(data, cc.var, X.vars, T.vars)
{
 
  data <- as.data.frame(data, stringsAsFactors=FALSE)
  nX   <- length(X.vars)

  data.nrow <- nrow(data)
  data.ncol <- ncol(data)
  rownames(data) <- as.character(1:data.nrow)

  X.mat <- as.matrix(removeOrKeepCols(data, X.vars, which=1))
  X.mat <-data.frame(X.mat)

  id.permute<-permut.res(X.mat)
  
  y.new<-data[id.permute, cc.var]
  

  data.new<-data
  data.new[, cc.var]<-y.new
  data.new[, T.vars]<-data[id.permute, T.vars]
  
  data.new
  
}

## only permute the genotype
permute.g.outcome<-function(data, cc.var, X.vars, T.vars)
{
 
  data <- as.data.frame(data, stringsAsFactors=FALSE)
  nX   <- length(X.vars)

  data.nrow <- nrow(data)
  data.ncol <- ncol(data)
  rownames(data) <- as.character(1:data.nrow)

  id.permute<-sample(c(1:data.nrow))

  data.new<-data
  data.new[, cc.var]<-data[id.permute, cc.var]
  data.new[, T.vars]<-data[id.permute, T.vars]
  data.new[, X.vars]<-data[id.permute, X.vars]
  
  data.new
  
}

resamp.outcome<-function(data, cc.var, X.vars, T.vars)
{
 
  data <- as.data.frame(data, stringsAsFactors=FALSE)
  nX   <- length(X.vars)

  data.nrow <- nrow(data)
  data.ncol <- ncol(data)
  rownames(data) <- as.character(1:data.nrow)

  X.mat <- as.matrix(removeOrKeepCols(data, X.vars, which=1))
  X.mat <-data.frame(X.mat)

  response <- rep.int(2, data.nrow)
  response[data[, cc.var]==1] <- 1
  model.1 <- vglm(response ~ -1 + ., family=multinomial, data=X.mat)
  off.1   <- model.1@predictors[, 1] 
  p.vec <-exp(off.1)/(1+exp(off.1))
  
  y.new<-rbinom(data.nrow, 1, p.vec)
  
  ## bootstrap
  
  id.vec<-c(1:data.nrow)

  id.newcase<-id.vec[y.new==1]
  id.newcn<-id.vec[y.new==0]
  n.newcase<-length(id.newcase)


  id.bm<-id.vec[data[,cc.var]==1]
  id.update<-sample(id.bm, size=n.newcase, replace=T)

  data.new<-data
  data.new[, cc.var]<-y.new

  data.new[id.newcase, T.vars]<-data[id.update, T.vars]
  data.new[id.newcn, T.vars]<-0

  data.new
  
}

## using the permuation from Qin's paper

boot.qin.outcome<-function(data, cc.var, X.vars, T.vars)
{
 
  data <- as.data.frame(data, stringsAsFactors=FALSE)
  nX   <- length(X.vars)

  data.nrow <- nrow(data)
  data.ncol <- ncol(data)
  rownames(data) <- as.character(1:data.nrow)

  X.mat <- as.matrix(removeOrKeepCols(data, X.vars, which=1))
  X.mat <-data.frame(X.mat)

  ## in vglm, the largest status is control
  response <- rep.int(2, data.nrow)
  response[data[, cc.var]==1] <- 1
  model.1 <- vglm(response ~ -1 + ., family=multinomial, data=X.mat)
  off.1   <- model.1@predictors[, 1] 
  
  n.cont<-sum(response==2)
  n.case<-data.nrow - n.cont

  ## distribution for control
  cont.dis<-1/(n.cont * (1 + exp(off.1)))
  case.dis<-cont.dis *exp(off.1)

  id.vec<-c(1:data.nrow)

  id.newcase<-sample(x=id.vec,size=n.case,  replace=T, prob=case.dis)
  id.newcont<-sample(x=id.vec,size=n.cont,  replace=T, prob=cont.dis)
	
  data.new<-data[c(id.newcase, id.newcont),]
  data.new[,cc.var]<-c(rep(1, n.case), rep(0, n.cont))

  id.bm<-id.vec[data[,cc.var]==1]
  ## permutation
  id.update<-sample(id.bm)

  data.new[1:n.case, T.vars]<-data[id.update, T.vars]
  data.new[(n.case+1):data.nrow, T.vars]<-0

  data.new
  
}


permtest.vglm<-function(data, cc.var, T.vars, X.vars, Z.vars, G.vars, wts, run.options)
{
    num.perm<-run.options$num.perm
    num.tree<-run.options$num.treemodel
    dev.mat<-matrix(0, ncol=num.tree, nrow=num.perm)
   
    if (run.options$method.tree.select == "merge")
    {
       dev.mat<-matrix(0, ncol=(num.tree-1), nrow=num.perm)
    }
    for (i in 1:num.perm)
    {
       if (run.options$permute == "restrict")
       {
          data.new<-restr.outcome(data, cc.var, X.vars, T.vars)
       }
       if (run.options$permute == "bootstrap")
       {
          data.new<-resamp.outcome(data, cc.var, X.vars, T.vars)
       }

       if (run.options$permute == "bootstrap.qin")
       {
          data.new<-boot.qin.outcome(data, cc.var, X.vars, T.vars)
       }

       if (run.options$permute == "permute.g")
       {
          data.new<-permute.g.outcome(data, cc.var, X.vars, T.vars)
       }

       dev.mat[i,]<-fitmodel(data.new, cc.var, T.vars, X.vars, Z.vars, G.vars,
                      wts, run.options)$dev.vec
    }       
    dev.mat
}


## new function
fitmodel <- function(data, cc.var, T.vars, X.vars, Z.vars, G.vars, wts, run.options)
{
  # data       Data frame of all data (including the intercept).
  # cc.var     Case-control variable (coded as 0 for controls and 1 for cases)
  # T.vars     Tumor variables names defined for cases. These must be factors!!!
  # X.vars     Homogeneous covariate names. Should include an intercept if desired.
  # Z.vars     Heterogeneous covariate names. Should include an intercept if desired.
  # G.vars     Genotype variables 
  # test.method	1 if using likelihood ratio test, 2 if Wald test

  ## setup options
  max.iter      <- run.options$max.iter
  converge.cut  <- run.options$converge.cut
  cart.option   <- run.options$cart.option
  mymethod      <- run.options$mymethod
  num.treemodel <- run.options$num.treemodel
  test.method <-run.options$test.method


  data <- as.data.frame(data, stringsAsFactors=FALSE)
  nX   <- length(X.vars)

  data.nrow <- nrow(data)
  data.ncol <- ncol(data)

  ## make T.vars to factor
  n.T<-length(T.vars)

  for (i in 1:n.T)
  {
    data[, T.vars[i]]<-factor(data[,T.vars[i]])
  }


  rownames(data) <- as.character(1:data.nrow)

  X.mat <- as.matrix(removeOrKeepCols(data, X.vars, which=1))
  X.mat <- data.frame(X.mat)

  # Add the offset
  offset.var <- "OFFSET_"
  data[, offset.var] <- NA
  
  # Global variables
  global.Z.cols     <<- Z.vars
  global.G.cols     <<- G.vars 
  global.controls   <<- data[, cc.var] %in% 0
  global.minbucket  <<- cart.option$minbucket
  global.offset.col <<- offset.var
  global.test	    <<- test.method
#  global.freq.cut   <<-run.options$freq.cut

  num.subj <- nrow(data)
  num.y.outcome <- 1

  # Get offset1 by calling vglm. Cases=1, controls = 2
  response <- rep.int(2, data.nrow)
  response[!global.controls] <- 1


  global.data <<- data

  # Variables that will be used for model 3 below 
  ZG.vars <- c(Z.vars, G.vars)

  # Get the data for rpart
  data.rpart  <- data[!global.controls, T.vars]
  y.rpart     <- as.integer(rownames(data.rpart))
  wts         <- wts[!global.controls]

  # Set the complexity parameter to be a negative value to allow the whole tree
  #   to be built
  cart.option$cp <- -1e100
	
  ## define a local function for calling rpart
  loopfun <- function() 
  {

	model.2 <- rpart(y.rpart ~ ., data=data.rpart, weights=wts,
                  method=mymethod, control=cart.option)
	model.2 <- editRpart(model.2)
		
	## only return the right submodel with the num.treemodel terminode
	sub.tree.result <- findsubtree(model.2, num.treemodel)
	
    # Recode the group.mat so that the values are 1, 2, ..., N instead of the node number
    mat <- sub.tree.result$group.mat 
    for (i in 1:nrow(mat)) {
      vec  <- mat[i, ]
      uvec <- sort(unique(vec), decreasing=FALSE)
      for (j in 1:length(uvec)) vec[vec == uvec[j]] <- j
      mat[i, ] <- vec
    }

loopout<-list(group.mat=mat, model.tree=model.2, 
 			num.diff.group=sub.tree.result$num.diff.group)

return(list(group.mat=mat, model.tree=model.2, 
 			num.diff.group=sub.tree.result$num.diff.group))
  }	 


  loopout <- loopfun()

  ### get the dev.vec
  dev.vec <- rep(0, num.treemodel)
  num.it  <- loopout$num.diff.group	

  ### change here
  if (dim(loopout$model.tree[[1]])[1] == 1)
  {
     ## the tree has no split 
	 warning("the overfitted tree is too small")
	 dev.vec    <- rep(0, num.treemodel)
     model.list <- list()
     var.name   <- 0
     nosplit.id <- 1
   } else {
       nosplit.id <- 0       
      ## need to update the group.mat
      if (run.options$method.tree.select == "bottom")
      {
        grp.tree <- loopout$group.mat[num.treemodel,]
       
        ## need to create the data.list here 
        data.list <- list(data=data, X.vars=X.vars, Z.vars=Z.vars, 
                         G.vars=G.vars, cc.var=cc.var)

        tt <- findsubtree.bottom(grp.tree, data.list, num.treemodel)
        loopout$group.mat <- tt$group.mat
        loopout$num.diff.group <- tt$num.diff.group
      }
    
  } 

  data.list <- list(data=data, X.vars=X.vars, Z.vars=Z.vars,       
                    G.vars=G.vars, cc.var=cc.var)
  if (run.options$method.tree.select == "merge")
  {
     stop("ERROR: can't use the merge option")
  }
  if (run.options$method.tree.select != "merge")
  {          
      dev.vec<-get.dev.top(loopout$group.mat, data.list, num.treemodel, test.method)
  }
  

  ### the smaller the more sigficant
  dev.vec <- (-1.0)*dev.vec

  return(list( model.tree=loopout$model.tree,
	 	group.mat=loopout$group.mat, 
                num.diff.group=loopout$num.diff.group, 	
                dev.vec=dev.vec, nosplit.id=nosplit.id))
}


get.dev.top<-function(group.mat, data.list, num.treemodel, test.method)
{
  # grp.tree        Output final partition from rpart
  # data.list       Named list of variables and data frame to be used with vglm
  #   X.vars
  #   Z.vars
  #   G.vars
  #   data          Data frame
  #   cc.var        Case-control variable
  # num.treemodel     Maximum number of iterations to combine nodes
  #                 to perform.

  data       <- data.list$data
  cases      <- data[, data.list$cc.var] %in% 1
  num.subj   <- nrow(data)

  X.vars     <- data.list$X.vars
  Z.vars     <- data.list$Z.vars
  G.vars     <- data.list$G.vars

  rm(data.list)
  temp <- gc(verbose=FALSE)

  dev.vec<-rep(0, num.treemodel)
  num.it<-length(unique(group.mat[num.treemodel,]))

  for (i in 1:num.it)
  {
     	grp.vec<-group.mat[i,]
     	response <- rep.int(0, num.subj)
     	response[cases] <- grp.vec
        dev.vec[i]<-GlobalTest(response,data[, G.vars])     
  }
  if (num.it < num.treemodel)
  {
	dev.vec[(num.it+1):num.treemodel]<-dev.vec[num.it]
  }
  dev.vec
}



## the returned gof.tot has no meaning for gee
findsubtree<-function(full.tree, num.of.tree)
{
	tree.table<-as.matrix(full.tree[[1]])
	num.subj<-length(full.tree[[2]])

	dev.vec<-as.numeric(tree.table[,4])
	var.vec<-tree.table[,1]
	node.id<-as.numeric(row.names(tree.table))
	
	num.of.node<-length(dev.vec)
	
	id.list<-c(1:num.of.node)	
	names(id.list)<-row.names(tree.table)

	
	## for each node, find out the increase of gof
	gofincrease<-rep(-1, num.of.node)
	left.node.id<-rep(-1, num.of.node)
	right.node.id<-rep(-1, num.of.node)

	term.node.vec<-grep("<", var.vec)
	inner.node.vec<-id.list[-term.node.vec]
	
	num.term.node<-length(term.node.vec)
	
	node.list.1<-node.id[inner.node.vec]*2
	node.list.2<-node.list.1+1

	id.1<-id.list[as.character(node.list.1)]
	id.2<-id.list[as.character(node.list.2)]

        ## other method to get the gof
	##gofincrease[inner.node.vec]<-dev.vec[inner.node.vec] -
	##		dev.vec[id.1] - dev.vec[id.2]


	gofincrease[inner.node.vec]<-dev.vec[inner.node.vec]

	left.node.id[inner.node.vec]<-id.1
	right.node.id[inner.node.vec]<-id.2

	num.it<-min(num.term.node, num.of.tree)

	tot.gof<-rep(0, num.of.tree)
	
	group.mat<-matrix(1, nrow=num.of.tree, ncol=num.subj)
	if (num.it == 1)
	{
		tot.gof[1:num.of.tree]<-dev.vec[id.list["1"]]

	}
	if (num.it == 2)
	{
		tot.gof[1]<-dev.vec[id.list["1"]]
		tot.gof[2:num.of.tree]<-sum(dev.vec[id.list[c("2","3")]])
		## get in the first two rows
		ances.mat<-findpartition(tree.table)
		## subtree's term node
		subtree.term<-id.list[c("2","3")]
		for (i in 1:2)
		{
			par.vec<-id.list[ances.mat[subtree.term[i],]==1]
			group.mat[2,is.element(full.tree[[2]], par.vec)]<-node.id[subtree.term[i]]
		}
		if (num.of.tree > 2)
		{
			group.mat[3:num.of.tree,]<-matrix(group.mat[2,], nrow=(num.of.tree-2), 
			ncol=num.subj, byrow=T)
		}
	}
	if (num.it > 2)
	{
		
		tot.gof[1]<-dev.vec[id.list["1"]]
		
		tot.gof[2]<-sum(dev.vec[id.list[c("2","3")]])

		ances.mat<-findpartition(tree.table)

		set.collect<-rep(-1, num.of.tree)
		set.collect[1:2]<-id.list[c("2","3")]	
		for (i in 1:2)
		{
			par.vec<-id.list[ances.mat[set.collect[i],]==1]
			group.mat[2,is.element(full.tree[[2]], par.vec)]<-node.id[set.collect[i]]
		}
	

		for (i in 2:(num.it-1))
		{
			cur.list<-set.collect[1:i]
			rank.1<-order(gofincrease[cur.list])[i]

			set.collect[rank.1]<-left.node.id[cur.list[rank.1]]
			set.collect[i+1]<-right.node.id[cur.list[rank.1]]

			
			tot.gof[i+1]<-sum(dev.vec[set.collect[1:(i+1)]])
			## find the grouping
			for (j in 1:(i+1))
			{
				par.vec<-id.list[ances.mat[set.collect[j],]==1]
				group.mat[i+1,is.element(full.tree[[2]], par.vec)]<-node.id[set.collect[j]]
			}

		}

		if (num.it < num.of.tree)
		{
			tot.gof[(num.it+1):num.of.tree]<-tot.gof[num.it]
			group.mat[(num.it+1):num.of.tree,]<-matrix(group.mat[num.it,], nrow=(num.of.tree-num.it), 
			ncol=num.subj, byrow=T)

		}
	}

	## based on group.mat, find the prediction for each person
##	pred.vec<-rep(0, num.subj)
	
##	m1<-as.character(group.mat[num.of.tree,])
##	pred.vec<-as.numeric(tree.table[m1,5])

	return(list(tot.gof=tot.gof, group.mat=group.mat, num.diff.group=num.it))
}	 			

## find subtree based on bottum up

findsubtree.bottom <- function(grp.tree, data.list, num.of.tree) {

  # grp.tree        Output object from rpart
  # data.list       Named list of variables and data frame to be used with vglm
  #   X.vars
  #   Z.vars
  #   G.vars
  #   data          Data frame
  #   cc.var        Case-control variable
  # num.of.tree     Maximum number of iterations to combine nodes
  #                 to perform.



 # Return the pair of nodes to combine
  # Get the cases in the data frame
  data       <- data.list$data
  cases      <- data[, data.list$cc.var] %in% 1
  num.subj   <- nrow(data)

  X.vars     <- data.list$X.vars
  Z.vars     <- data.list$Z.vars
  G.vars     <- data.list$G.vars

  # Get the vector containing the leaf node id for each observation
  where <- grp.tree
  names(where) <- NULL

  rm(data.list)
  temp <- gc(verbose=FALSE)

  # Initialize
  where.uniq <- sort(unique(where))
  n.nodes    <- length(where.uniq)

  ## has to start from the bottom
  num.it <- n.nodes

  # Initialize the matrix to hold the node numbers from combining nodes
  group.mat  <- matrix(data=NA, nrow=num.it, ncol=length(where))

  # Set the first row of group.mat to be the original output from rpart
  group.mat[1, ] <- where

  if (num.it > 2)
  {   

    # The baseline node will be n.nodes + 1 (for the controls). This should not be needed.
    base <- n.nodes + 1

    for (iter in 2:num.it) {
      # Get the pair of nodes to combine
           response <- rep.int(0, num.subj)
           response[cases] <- where
           nodes<-GroupMerge(response, data[,G.vars])

      if (is.null(nodes)) {
        group.mat <- group.mat[1:(iter-1), ]
        break
      }

      # Update. Change the larger node number to be the smaller one, and
      # change so that the where vector is still 1, 2, ..., n
      maxnode           <- max(nodes)
      temp              <- (where == maxnode)
      where[temp]       <- min(nodes)
      temp              <- (where > maxnode)
      where[temp]       <- where[temp] - 1
      group.mat[iter, ] <- where
      where.uniq        <- sort(unique(where))
      n.nodes           <- length(where.uniq)   
      if (n.nodes < 3) break
    } # END: for (iter in 2:num.it)
     
    ## the last row is 1's
    group.mat[num.it, ] <- 1
 
    ## reverse
    n.row<-nrow(group.mat)
    n.col<-ncol(group.mat)

    group.mat<-group.mat[rev(c(1:n.row)),]

    if (num.of.tree <= n.row)
    {
      group.mat<-group.mat[1:num.of.tree,]
      num.diff.group<-num.of.tree
    }
    if (num.of.tree > n.row)     
    {
      grp.more<-matrix(group.mat[n.row,], byrow=T, 
               ncol=n.col,  nrow=(num.of.tree-n.row))
      group.mat<-rbind(group.mat, grp.more)
      num.diff.group<-n.row
    }
  } else {
    ## the last row is 1's
    group.mat[num.it, ] <- 1

    ## assume num.of.tree alwasys larger than 2
    #group.mat<-matrix(1, nrow=num.of.tree, ncol=length(where))
    group.mat[(num.it+1):num.of.tree,]<-matrix(where, byrow=T, 
        ncol=length(where), nrow=(num.of.tree-num.it-1))

    num.diff.group<-num.it+1
  }    
        
  list(group.mat=group.mat, num.diff.group=num.diff.group)
  
} # END: findsubtree.bottom




findpartition<-function(tree.table)
{
	var.vec<-tree.table[,1]
	node.id<-as.numeric(row.names(tree.table))
	
	num.of.node<-length(node.id)
	if (num.of.node < 3)
	{
		stop("the tree is too samll")
	}		
	id.list<-c(1:num.of.node)	
		
	term.node.vec<-grep("<", var.vec)	
	num.term.node<-length(term.node.vec)

	mat.term<-matrix(-1, nrow=num.of.node, ncol=num.of.node)
	for (i in 1: num.term.node)
	{
		col.id<-term.node.vec[i]
		t1<-findparentid(node.id[col.id])
		t1<-as.character(t1)
		t2<-mat.term[, col.id]
		names(t2)<-row.names(tree.table)
		t2[t1]<-1
		mat.term[,col.id]<-t2
	}
	return(mat.term)
}

findparentid<-function(term.id)
{
	ances.vec<-rep(-1, 1000)
	if (term.id < 2)
	{
		stop("wrong with terminal node id \n")
	}
	id.step<-term.id
	my.index<-2
	ances.vec[1]<-term.id
	while(id.step != 1)
	{
		id.step<-id.step%/%2
		ances.vec[my.index]<-id.step
		my.index<-my.index+1
	}

	my.index<-my.index-1
	return(ances.vec[1:my.index])
}	


get.pvalue.check<-function(dev.mat, dev.obs)
{
	num.tree<-dim(dev.mat)[2]
	num.perm<-dim(dev.mat)[1]
	num.perm1<-num.perm
	p.obs<-rep(0, num.tree)

	## find obs p
	for (i in 1:num.tree)
	{
		t1<-dev.mat[,i][1:num.perm1]
		a<-length(t1[t1<= dev.obs[i]])
		p.obs[i]<-(a+1)/(num.perm1+1)
	}
	minp.obs<-min(p.obs[2:num.tree])
	p.for.minp<-0.99999
	return(list(pobs=p.obs, minp.obs=minp.obs, p.for.minp=p.for.minp))
}

get.pvalue.old<-function(dev.mat, dev.obs)
{
	num.tree<-dim(dev.mat)[2]
	num.perm<-dim(dev.mat)[1]
	num.perm1<-num.perm/2
	num.perm2<-num.perm - num.perm1
	p.obs<-rep(0, num.tree)

	## find obs p
	for (i in 1:num.tree)
	{
		t1<-dev.mat[,i][1:num.perm1]
		a<-length(t1[t1<= dev.obs[i]])
		p.obs[i]<-(a+1)/(num.perm1+1)
	}
	minp.obs<-min(p.obs[2:num.tree])

	if (num.tree == 1)
	{
		p.for.minp<-minp.obs
	}

	## find the p for minp, don't count the first column

	if (num.tree >= 2)
	{
		pmat<-matrix(0, ncol=(num.tree-1), nrow=num.perm2)

		for (i in 1: (num.tree-1))
		{
			t1<-dev.mat[,i+1][(num.perm1+1):num.perm]
			pmat[,i]<-rank(t1, ties.method="random")/num.perm2
		}
		minp.vec<-apply(pmat,1, min)

		p.for.minp<-(length(minp.vec[minp.vec<= minp.obs]) + 1)/(1+num.perm2)
	}
	
	return(list(pobs=p.obs, minp.obs=minp.obs, p.for.minp=p.for.minp))
}

## the first column is the test from chisq, we can get the pvalue directly

get.pvalue.1<-function(dev.mat, dev.obs)
{
        num.test<-length(dev.obs)
        if (num.test == 1)
        {
           dev.mat<-matrix(dev.mat, ncol=1, byrow=T)
        }
        
        num.perm<-dim(dev.mat)[1]
     
        ## the first is the observed, the rest is the permuted 
        pmat<-matrix(0, ncol=num.test, nrow=(num.perm+1))
               
        dev.mat<-rbind(dev.obs, dev.mat)
        my.n<-num.perm + 1

        for (i in 1: num.test)
        {       
            ## the larger the more significant

            t1<-(-1.0) * dev.mat[,i]
            pmat[,i]<-rank(t1, ties.method="random")/my.n
        }
        
        pmat[,1]<-pchisq(dev.mat[,1], df=1, lower.tail=FALSE)

        p.obs<-pmat[1,]       
        
        if (num.test == 1)
        {
          p.for.minp.perm.all<-pmat[,1]   
        }
        
        if (num.test > 1)
        {
          a<-apply(pmat, 1, min)
          p.for.minp.perm.all<-rank(a, ties.method="random")/my.n
        }          
        
        return(list(p.obs=p.obs,  p.for.minp=p.for.minp.perm.all[1]))
}


get.pvalue<-function(dev.mat, dev.obs)
{
        num.test<-length(dev.obs)
        if (num.test == 1)
        {
           dev.mat<-matrix(dev.mat, ncol=1, byrow=T)
        }
        
        num.perm<-dim(dev.mat)[1]
     
        ## the first is the observed, the rest is the permuted 
        pmat<-matrix(0, ncol=num.test, nrow=(num.perm+1))
               
        dev.mat<-rbind(dev.obs, dev.mat)
        my.n<-num.perm + 1

        for (i in 1: num.test)
        {       
            ## the larger the more significant

            t1<-(-1.0) * dev.mat[,i]
            pmat[,i]<-rank(t1, ties.method="random")/my.n
        }


        p.obs<-pmat[1,]       
        
        if (num.test == 1)
        {
          p.for.minp.perm.all<-pmat[,1]   
        }
        
        if (num.test > 1)
        {
          a<-apply(pmat, 1, min)
          p.for.minp.perm.all<-rank(a, ties.method="random")/my.n
        }          
        
        return(list(p.obs=p.obs,  p.for.minp=p.for.minp.perm.all[1]))
}

strat.def<-function(strat.id)
{
   
    cut.val<-5
    freq.1<-table(strat.id)
    
    name.1<-as.numeric(names(freq.1))

    num.s<-length(freq.1)

    if (max(freq.1) < 5)
    {
      stop("strat size is too samll \n")
    }
   
    strat.out<-strat.id

    s.max<-name.1[freq.1 == max(freq.1)]
    s.max<-s.max[1]

    for (i in 1:num.s)
    {
       if (freq.1[i] < cut.val)
       {
          strat.out[strat.id == name.1[i]]<-s.max
       }
    }
    strat.out
}
           
 
# Function to remove columns or variables from a matrix or data frame
removeOrKeepCols <- function(x, vars, which=1, stopOnError=1) {

  # x      Matrix or data frame 
  # vars   Vector of variable names or column numbers.
  # which  1 or -1 to keep or remove cols
  #        The default is 1 
  # stopOnError  0 or 1 to call stop() if columns are not found

  n      <- dim(x)
  dfFlag <- is.data.frame(x)
  if (is.null(n)) stop("ERROR: x should be 2 dimensional")
  if (which != 1) which <- -1

  if (!is.numeric(vars)) {
    cols <- match(vars, colnames(x))
    temp <- is.na(cols)
    if (any(temp)) {
      if (stopOnError) {
        print(vars[temp])
        print("The above columns were not found in the data")
        stop("ERROR in removeOrKeepCols")
      }
      cols <- cols[!temp]
      if (!length(cols)) {
        if (which == 1) {
          return(NULL)
        } else {
          return(x)
        }
      }
    }
  } else {
    cols <- vars
  }
  cols <- which*cols

  # Keep or remove columns
  ret <- x[, cols]
  
  # Check for NULL dimension
  if (is.null(dim(ret))) {
    dim(ret) <- c(n[1], length(ret)/n[1])
    if (dfFlag) ret <- data.frame(ret)
    cnames <- colnames(x)
    if (!is.null(cnames)) colnames(ret) <- cnames[cols]
    rownames(ret) <- rownames(x)
  }

  ret

} # END: removeOrKeepCols

