% --- Source file: het.tree.fast.Rd ---
\name{het.tree.fast}
\alias{het.tree.fast}
\title{het.tree.fast}
\description{
  A robust association test for detecting genetic variants with heterogeneous effects 
}
\usage{
het.tree.fast(data, cc.var, T.vars, X.vars, Z.vars, G.vars, wts, run.options,
                        Robj.file, myseed) 
}
\arguments{
  \item{data}{A data frame containing all data including an intercept column.}
  \item{cc.var}{The name of the case-control variable coded as 0-1 (0=control, 1=case). }
  \item{T.vars}{A character vector of variables used to build the tree. }
  \item{X.vars}{A character vector of variables with homogeneous effects. }
  \item{Z.vars}{A character vector of variables with heterogeneous effects. }
  \item{G.vars}{A character vector of genetic variables coded as 0-1-2. }
  \item{wts}{A numeric vector of weights. }
  \item{run.options}{List of options (see details). }
  \item{Robj.file}{Output file created with the \code{\link[base]{save}} function. }
  \item{myseed}{Random seed. }
}
\value{
  The returned object is NULL. The output is saved in the R object file \code{Robj.file}.
 }

\details{  
   
  \bold{run.options list:} \cr
 Below are the names for the options list \code{run.options}. All must be specified.
\itemize{
  \item{\code{cart.option}}{ List of options for \code{\link[rpart]{rpart}}. 
                             See \code{\link[rpart]{rpart.control}}. }
  \item{\code{converge.cut}}{ Numeric value}
  \item{\code{num.treemodel}}{ Integer}
  \item{\code{max.iter}}{ Integer}
  \item{\code{num.perm}}{ Integer}
  \item{\code{method.tree.select}}{ Character string "top"}
  \item{\code{permute}}{ Character string "restrict.xz"}
  \item{\code{test.method}}{ 1 or 2. 1=LLR test, 2=Wald test}
} % END: itemize


} % END: details

\references{ 
Yu K, Zhang H, Wheeler W, Home HN, Chen J, Figueroa JD. 
 A robust association test for detecting genetic variants with heterogeneous effects (submitted)
}
%\author{Han Zhang and Kai Yu }
%\seealso{ \code{\link{treat}} }
\examples{

  myseed     <- 1 
  inputfile  <- system.file("sampleData", "data_1", package="Het.Tree.Fast")
  outputfile <- paste(getwd(), "/out.rda", sep="")

  data   <- read.table(inputfile,  header=1)
  cc.var <- "y.all"
  T.vars <- c("t1", "t2", "t3",  "t4", "t5")

  # The intercept is first (for X.vars), also, only the same intercept variable
  #  should be used
  X.vars <- c("int.x")
  Z.vars <- c("int.x")
  G.vars <- c("g.all")

  wts          <- rep(1, nrow(data))
  test.method  <- 2
  xval         <- 0
  minsplit     <- 50
  maxdepth     <- 5
  maxcompete   <- 0
  maxsurrogate <- 0
  usesurrogate <- 2
  minbucket    <- 20
  cart.option  <- rpart.control(xval=xval, minbucket=minbucket, 
                  minsplit=minsplit, maxdepth=maxdepth, maxcompete=maxcompete, 
                  maxsurrogate=maxsurrogate, usesurrogate=usesurrogate) 
converge.cut       <- 0.001
num.treemodel      <- 5
max.iter           <- 2
num.perm           <- 500
permute.method     <- "permute.g"
method.tree.select <- "top"

run.options <- list(converge.cut=converge.cut,
           num.treemodel=num.treemodel, 
           max.iter=max.iter,
           method.tree.select=method.tree.select, 
           permute = permute.method,
           test.method = test.method,
           num.perm=num.perm, cart.option=cart.option) 
 
# Main function
het.tree.fast(data, cc.var, T.vars, X.vars, Z.vars, G.vars, 
           wts, run.options, outputfile, myseed)


load(outputfile)

## the test result
obj$test.result

## the full tree
obj[[1]][[1]]

}

\keyword{model} 

