//this file is re-organized from multinomial.cpp for release purpose
//the functions implemented here will be used in Kai Yu's revised paper
//the trend test, which has close form, is used for group merging and splitting
//in the trend test, the data from control group is discarded
//I therefore removed all feature from the numerical recipes library
//and removed two functions based on the multinomial regression model

#include "TrendInTree.hpp"

extern "C" {

//grp_label: 0, 1, ..., K. 0 is the control group
//grp_num: number of case groups, i.e K
//n: sample size
//geno: genotypes, coded as 0, 1, 2
void GlobalTest(const int *const grp_label, const int *const grp_num, const int *const N, const int *const geno, double *stat, int *dof){

	vector<vector<int> > cnt(*grp_num+1, vector<int> (3, 0)); // n_{group, genotype}, first row is control group
	vector<int> m(3, 0);
	for(int i = 0; i < *N; ++i){
		if(geno[i] < 0 || geno[i] > 2){
			cout << "The genotype must be coded as 0, 1 and 2" << endl;
			exit(1);
		}
		if(grp_label[i] < 0 || grp_label[i] > *grp_num){
			cout << "Invalid group label of outcome" << endl;
			exit(1);
		}
		cnt[grp_label[i]][geno[i]]++;
		m[geno[i]]++;
	}

	vector<int> n(*grp_num + 1, 0);
	for(int k = 0; k < *grp_num +1; ++k){
		n[k] = cnt[k][0] + cnt[k][1] + cnt[k][2];
	}

	double Eg = (m[1] + 2.0 * m[2])/(*N);
	double Eg2 = (m[1] + 4.0 * m[2])/(*N);
	vector<double> score(*grp_num, .0);
	double sum_score = .0;
	double sum_score2 = .0;
	for(int k = 0; k < *grp_num; ++k){
		score[k] = cnt[k+1][1] + 2*cnt[k+1][2] - n[k+1] * Eg;
		sum_score += score[k];
		sum_score2 += score[k]*score[k]/n[k+1];
	}

	double c = 1.0/(Eg2 - Eg * Eg);
	*stat = c * ((sum_score * sum_score)/n[0] + sum_score2);
	*dof = *grp_num;

}

}


extern "C" {

//grp_label: 0, 1, ..., K. 0 is the control group
//grp_num: number of case groups, i.e K
//N: sample size
//geno: genotypes


void GroupMerge_trend(const int *const grp_label, const int *const grp_num, const int *const N, const int *const geno, double *stat, int *idx_grp1, int *idx_grp2){

	vector<vector<int> > cnt0(*grp_num + 1, vector<int>(3, 0)); // n_{group, genotype}, first row is control group
	for(int i = 0; i < *N; ++i){
		if(geno[i] < 0 || geno[i] > 2){
			cout << "The genotype must be coded as 0, 1 and 2" << endl;
			exit(1);
		}
		if(grp_label[i] < 0 || grp_label[i] > *grp_num){
			cout << "Invalid group label of outcome" << endl;
			exit(1);
		}
		cnt0[grp_label[i]][geno[i]]++;
	}

	vector<int> n(*grp_num + 1, 0);
	for(int k = 0; k < *grp_num +1; ++k){
		n[k] = cnt0[k][0] + cnt0[k][1] + cnt0[k][2];
	}

	int idx = -1;
	for(int k = 1; k < *grp_num; ++k){
		for(int l = k+1; l < *grp_num + 1; ++l){
			idx++;

			idx_grp1[idx] = k;
			idx_grp2[idx] = l;

			int m[3] = {0};
			for(int i = 0; i < 3; ++i){
				m[i] = cnt0[k][i] + cnt0[l][i];
			}

			int n12 = n[k] + n[l];

			double myg = (cnt0[l][1] + 2.0*cnt0[l][2])/n12;
			double my = n[l] * 1.0/n12;
			double mg = (m[1] + 2.0*m[2])/n12;
			double mg2 = (m[1] + 4.0*m[2])/n12;
			stat[idx] = n12 * (myg - my * mg) * (myg - my * mg) / my / (1.0-my)/(mg2-mg*mg);

		}
	}

}


}

extern "C" {

//grp_label: 0, 1, 2. 0 is the control group
//N: sample size
//geno: genotypes
//stat: statistics of 1 or 2 degree of freedom
//dof: degree of freedom


void GroupSplit_trend(const int *const grp_label, const int *const N, const int *const geno, double *stat){

	int cnt0[2][3] = {0}; // n_{group, genotype}, group = 1/2
	for(int i = 0; i < *N; ++i){
		if(geno[i] < 0 || geno[i] > 2){
			cout << "The genotype must be coded as 0, 1 and 2" << endl;
			exit(1);
		}
		if(grp_label[i] < 0 || grp_label[i] > 2){
			cout << "Invalid group label of outcome" << endl;
			exit(1);
		}

		if(grp_label[i] != 0){
			cnt0[grp_label[i]-1][geno[i]]++;
		}
	}

	int n[2] = {0};
	for(int k = 0; k < 2; ++k){
		n[k] = cnt0[k][0] + cnt0[k][1] + cnt0[k][2];
	}

	int m[3] = {0};
	for(int k = 0; k < 3; ++k){
		m[k] = cnt0[0][k] + cnt0[1][k];
	}

	int n12 = n[0] + n[1];
	double myg = (cnt0[1][1] + 2.0 * cnt0[1][2])/n12;
	double my = n[1] * 1.0/n12;
	double mg = (m[1]+2.0*m[2])/n12;
	double mg2 = (m[1]+4.0*m[2])/n12;

	*stat = n12 * (myg - my * mg) * (myg - my * mg) / my / (1.0-my)/(mg2-mg*mg);

}


}

